package net.sf.itcb.addons.monitoring.track;

import java.util.UUID;

import org.slf4j.MDC;

/**
 * This class gets the transaction id in the current thread or generates it if it doesn't exist<br/>
 * The transaction ID is stored in ThreadLocal.<br/>
 * In order to trace it in logs, it is stored with MDC associated to the key : <code>itcbTransactionId</code>
 * @author Pierre Le Roux
 *
 */
public class TransactionIdHolder {
	
	/**
	 * Transaction ID in log files using <code>%X{itcbTransactionId}</code> in log4j.xml patterns (for example).
	 */
	 public static final String MDC_TRANSACTION_ID = "itcbTransactionId";
	
	 /**
	  * the transactionId management
	  */
	 private static ThreadLocal < String > transactionId = new ThreadLocal < String > () {
            @Override protected String initialValue() {
                return null;
            }

	        @Override
			public String get() {
				if(super.get() != null) {
					return super.get();
				}
				else {
					String id = UUID.randomUUID().toString();
					set(id);
					MDC.put(MDC_TRANSACTION_ID, id);
					return id;
				}
	        }
	 };
	 
	 /**
	  * Set the transaction Id in ThreadLocal : 
	  * usefull when we received a message from another service with an already created transactionId
	  * @param transactionId
	  */
	 public static void setTransactionId(String transactionId) {
		 TransactionIdHolder.transactionId.set(transactionId);
	 }
	 
	 /**
	  * get the transactionId in threadlocal
	  * @return the transaction id
	  */
	 public static String getTransactionId() {
		 return TransactionIdHolder.transactionId.get();
	 }
}

