package net.sf.itcb.addons.monitoring.track.client;

import net.sf.itcb.addons.monitoring.track.Globals;
import net.sf.itcb.addons.monitoring.track.TransactionIdHolder;

import org.springframework.ws.client.WebServiceClientException;
import org.springframework.ws.client.support.interceptor.ClientInterceptor;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.soap.SoapMessage;

/**
 * Client interceptor that put transactionId in soap headers.<br/>
 * It is needed in order to give transactionId to the server.<br/>
 * 
 * <i>Example Configuration in spring context file :</i><br/>
 * <code>
 *   &lt;bean id="yourCompanyWebserviceTemplate" class="org.springframework.ws.client.core.WebServiceTemplate" scope="prototype"&gt;<br/>
 *       &lt;property name="interceptors"&gt;<br/>
 *		    &lt;list&gt;<br/>
 *		      &lt;!-- <ref local="wsSecurityInterceptor" /&gt;--&gt;<br/>
 *		      &lt;ref bean="trackerClientInterceptor" /&gt;<br/>
 *		    &lt;/list&gt;<br/>
 *		&lt;/property&gt;<br/>
 *		&lt;property name="faultMessageResolver" ref="itcbFaultMessageResolver"/&gt;<br/>
 *   &lt;/bean&gt;<br/>
 *  <br/>
 * 
 * &lt;bean id="trackerClientInterceptor" class="net.sf.itcb.addons.monitoring.track.client.TrackerClientInterceptor"/&gt;
 * </code>
 * <br/><br/>
 * 
 * This configuration should be done in company client common spring files in order to be shared by all you IT webservice clients.
 * @author Pierre Le Roux
 *
 */
public class TrackerClientInterceptor implements ClientInterceptor {
	
	@Override
	public boolean handleFault(MessageContext arg0)
			throws WebServiceClientException {		
		return true;
	}

	/**
	 * Store the transaction ID in soap headers
	 */
	@Override
	public boolean handleRequest(MessageContext messageContext)
			throws WebServiceClientException {
	    ((SoapMessage) messageContext.getRequest()).getSoapHeader().addAttribute(Globals.HEADER_TRANSACTION_ID_QNAME , TransactionIdHolder.getTransactionId());
	    return true;
	}

	@Override
	public boolean handleResponse(MessageContext messageContext)
			throws WebServiceClientException {
		return true;
	}

}
